/*
 * Sudoku Solver Copyright (C) 2005 Keith Flanagan
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * This class does all the real work involved in solving a sudoku puzzle.
 * 
 * @author Keith Flanagan
 */
public class Solver
{
  private byte[][] initialGrid;

  private int subgridSize;

  private static final byte EMPTY_VAL = 0;

  private static final byte START = 1;

  private static byte END;

  public Solver()
  {
  }

  public Solver(byte[][] grid)
  {
    setInitialGrid(grid);
  }

  public byte[][] getInitialGrid()
  {
    return initialGrid;
  }

  public void setInitialGrid(byte[][] initialGrid)
  {
    this.initialGrid = initialGrid;
    subgridSize = (int) Math.sqrt(initialGrid.length);
    END = (byte) initialGrid.length;
  }

  /**
   * Given an initial grid, this method sets about finding a solution to the
   * given puzzle.
   * 
   * @return a list of valid solutions.
   */
  public List<byte[][]> solve()
  {
    final int gameDim = initialGrid.length;
    final Set<Byte> alphabet = new HashSet<Byte>();
    // Define the "alphabet" - the allowable values each cell can have
    for (byte v = START; v <= END; v++) {
      alphabet.add(v);
    }
    System.out.println("Alphabet size: " + alphabet.size());
    printGrid(initialGrid);

    // Cast in following line is workaround for java bug id: 6229728
    final Set<Byte>[][] allowedValues = (Set<Byte>[][]) new HashSet[gameDim][gameDim];
    for (int y = 0; y < gameDim; y++) {
      for (int x = 0; x < gameDim; x++) {
        Set<Byte> allowedCellValues = new HashSet<Byte>();
        allowedValues[y][x] = allowedCellValues;
        // If the cell is one of our starting clues, then its initial value is
        // the only one it may take
        if (initialGrid[y][x] != EMPTY_VAL) {
          allowedCellValues.add(initialGrid[y][x]);
          continue;
        }
        // ...otherwise, initially allow all elements of the alphabet as valid
        // entries for this cell ...
        allowedCellValues.addAll(alphabet);
        // ... and then remove invalid values accordingly
        Set<Byte> toRemove = new HashSet<Byte>();
        for (byte toCheck : allowedCellValues) {
          byte oldVal = initialGrid[y][x];
          initialGrid[y][x] = toCheck;
          if (!testCellValid(initialGrid, y, x)) {
            toRemove.add(toCheck);
          }
          initialGrid[y][x] = oldVal;
        }
        allowedCellValues.removeAll(toRemove);
      }
    }

    /*
     * At this point we have a set of "allowable" values that each cell can take
     * without violating the rules of the game. This has hopefully reduced the
     * search space significantly, so we can now attempt to solve the puzzle.
     */
    List<byte[][]> solutions = new LinkedList<byte[][]>();

    guessGrid(gridCopy(initialGrid), 0, 0, allowedValues, solutions);
    return solutions;
  }

  private void guessGrid(byte[][] grid, int x, int y,
      Set<Byte>[][] allowedValues, List<byte[][]> solutions)
  {
    while (grid[y][x] != EMPTY_VAL) {
      if (!testCellValid(grid, y, x)) {
        // There's no point going any further if this cell is not valid ...
        // ... although we shouldn't actually get here
        return;
      }
      x++;
      if (x == grid.length) {
        x = 0;
        y++;
      }
      if (y == grid.length) {
        // We've finished
        solutions.add(grid);
        return;
      }
    }

    int nextX = x;
    int nextY = y;
    nextX++;
    if (nextX == grid.length) {
      nextX = 0;
      nextY++;
    }
    for (byte val : allowedValues[y][x]) {
      grid[y][x] = val;
      // FIXME - for efficieny, remove this val from other cells in this row,
      // col, subgrid. To do this, make a copy of allowedValues, and only modify
      // the copy. Make sure to send the COPY to future recursive calls

      if (!testCellValid(grid, y, x)) {
        continue;
      }
      if (nextY == grid.length) {
        // We've finished, and found that this is a solution!
        solutions.add(grid);
        return;
      }
      // If we're not finished, continue the search. We know that everything up
      // to cell (x,y) is valid so far
      guessGrid(gridCopy(grid), nextX, nextY, allowedValues, solutions);
    }
  }

  /**
   * Given a game grid, and a cell coordinate, checks to see if the value at
   * that coordinate appears anywhere else within the same row, column, or
   * subgrid. True is returned if the value is not found elsewhere (ie, the cell
   * is valid). If the cell is not valid, false is returned.
   * 
   * @param grid
   *          the grid to test
   * @param row
   *          coordinate of cell to test for validity
   * @param col
   *          coordinate of cell to test for validity
   * @return true if the value in the specified cell is valid placement
   *         according to the rules of Sudoku, otherwise, false.
   */
  public boolean testCellValid(byte[][] grid, int row, int col)
  {
    int coordVal = grid[row][col];

    // Check grid row contents for our proposed value
    for (int x = 0; x < grid.length; x++) {
      if (x == col) {
        continue;
      }
      if (grid[row][x] == coordVal) {
        return false;
      }
    }

    // Check grid column contents for our proposed value
    for (int y = 0; y < grid.length; y++) {
      if (y == row) {
        continue;
      }
      if (grid[y][col] == coordVal) {
        return false;
      }
    }

    // Check sub-grid for any cells containing our proposed value
    // These are calculated outside the loop for performance reasons
    int yStart = (row / subgridSize) * subgridSize;
    int yEnd = yStart + subgridSize;
    int xStart = (col / subgridSize) * subgridSize;
    int xEnd = xStart + subgridSize;

    for (int y = yStart; y < yEnd; y++) {
      for (int x = xStart; x < xEnd; x++) {
        if ((x == col) && (y == row)) {
          continue;
        }
        if (grid[y][x] == coordVal) {
          return false;
        }
      }
    }
    // If we've reached this point, then value was not found on the row, column,
    // or sub-grid we're interested in.
    return true;
  }

  /**
   * Copies a grid
   * 
   * @param toCopy
   *          grid to copy
   * @return copy of <code>toCopy</code>
   */
  public static byte[][] gridCopy(byte[][] toCopy)
  {
    byte[][] copy = new byte[toCopy.length][toCopy.length];
    for (int y = 0; y < toCopy.length; y++) {
      System.arraycopy(toCopy[y], 0, copy[y], 0, toCopy.length);
    }
    return copy;
  }

  /**
   * Prints a grid. Used for debugging.
   * 
   * @param grid
   *          the grid to print
   */
  public static void printGrid(byte[][] grid)
  {
    for (int y = 0; y < grid.length; y++) {
      for (int x = 0; x < grid.length; x++) {
        System.out.print(grid[y][x] + ", ");
      }
      System.out.println();
    }
  }
}
