/*
 * Sudoku Solver Copyright (C) 2005 Keith Flanagan
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.border.BevelBorder;

/**
 * This is the "main" user interface. This class sets up a gui displaying a
 * grid. Options are provided for solving, loading and saving grids.
 * 
 * @author Keith Flanagan
 */
public class Sudoku extends JPanel
{
  private JFrame owner;

  private final Solver solver;

  private JToolBar toolBox;

  private GridPanel gridPanel;

  private byte[][] currentGrid;

  public Sudoku(byte[][] initialGrid)
  {
    currentGrid = initialGrid;
    solver = new Solver();
    toolBox = new SudokuToolbox(this);
    gridPanel = new GridPanel(currentGrid);
    setLayout(new BorderLayout());
    add(toolBox, BorderLayout.NORTH);
    add(gridPanel, BorderLayout.CENTER);
  }

  public Sudoku()
  {
    this(new byte[9][9]);
  }

  public void solve() throws InvalidGridException
  {
    solver.setInitialGrid(currentGrid);
    List<byte[][]> solutions = solver.solve();
    if (solutions.size() == 0) {
      throw new InvalidGridException(InvalidGridException.NONE_FOUND);
    } else if (solutions.size() > 1) {
      throw new InvalidGridException(InvalidGridException.MULTIPLE_FOUND,
          solutions);
    }

    GridPanel solutionPanel = new GridPanel(solutions.get(0));

    JFrame solutionFrame = new JFrame();
    solutionFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    int x = this.getBounds().x + 50;
    int y = this.getBounds().y + 50;
    solutionFrame.setBounds(x, y, x + 350, y + 475);
    solutionFrame.setSize(350, 475);
    solutionFrame.getContentPane().setLayout(new BorderLayout());
    solutionFrame.getContentPane().add(solutionPanel);
    solutionFrame.setTitle("Solution");
    solutionFrame.setVisible(true);
  }

  public void setOwner(JFrame owner)
  {
    this.owner = owner;
  }

  public JFrame getOwner()
  {
    return owner;
  }

  public byte[][] getCurrentGrid()
  {
    return currentGrid;
  }

  public void setCurrentGrid(byte[][] grid)
  {
    this.currentGrid = grid;
    gridPanel.setGrid(currentGrid);
    gridPanel.redraw();
    gridPanel.revalidate();
  }

  private class GridPanel extends JPanel
  {
    private byte[][] grid;

    public GridPanel(byte[][] grid)
    {
      setGrid(grid);
    }

    public void redraw()
    {
      removeAll();
      setLayout(new GridLayout(grid.length, grid.length));
      for (int y = 0; y < grid.length; y++) {
        for (int x = 0; x < grid.length; x++) {
          JLabel label = new JLabel();
          label.setBorder(new BevelBorder(BevelBorder.RAISED));
          if (grid[y][x] != 0) {
//            char asciiChar = (char) grid[y][x];
//            asciiChar += 48;
//            label.setText("" + asciiChar);
            label.setText(String.valueOf(grid[y][x]));
            label.setHorizontalAlignment(JLabel.CENTER);
          }
          add(label);
        }
      }
    }

    public void setGrid(byte[][] grid)
    {
      this.grid = grid;
      redraw();
    }
  }

  /**
   * @param args
   */
  public static void main(String[] args)
  {
//    Map<Byte, Character> alphabet = new HashMap<Byte, Character>();
//    int element = 1;
//    for (int i = 1; i < 10; i++) {
//      alphabet.put((byte) i, (char) i);
//    }

    Sudoku panel = new Sudoku(new byte[9][9]);

    JFrame frame = new JFrame();
    panel.setOwner(frame);
    frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    frame.setSize(350, 475);
    frame.getContentPane().setLayout(new BorderLayout());
    frame.getContentPane().add(panel);
    frame.setTitle("Sudoku Solver");
    frame.setVisible(true);
  }
}
